@@ -11,6 +11,7 @@ __PACKAGE__->mk_accessors(qw/aws_secret_access_key aws_access_key_id token regio
 			     serialized logger
 			    /);
 
+use Carp;
 use Data::Dumper;
 use Date::Format;
 use Exporter;
@@ -27,7 +28,7 @@ use constant  AWS_CONTAINER_CREDENTIALS_URL          => 'http://169.254.170.2';
 
 use vars qw/$VERSION @EXPORT/;
 
-$VERSION = '1.0.10-5'; $VERSION=~s/\-.*$//;
+$VERSION = '1.0.11'; $VERSION=~s/\-.*$//;
 
 @EXPORT = qw/$VERSION/;
 
@@ -50,7 +51,7 @@ C<AWS::Credentials>
 
 =head1 SYNOPSIS
 
- my $aws_creds = new Amazon::Credentials({order => [qw/env file container role/]});
+ my $aws_creds = Amazon::Credentials->new({order => [qw/env file container role/]});
 
 =head1 DESCRIPTION
 
@@ -68,7 +69,7 @@ meta-data>. See L</new>.
 
  new( options );
 
- my $aws_creds = new Amazon::Credential( { profile => 'sandbox', debug => 1 });
+ my $aws_creds = Amazon::Credential->new( { profile => 'sandbox', debug => 1 });
 
 C<options> is a hash of keys that represent options.  Any of the
 options can also be retrieved using their corresponding 'get_{option}
@@ -108,7 +109,7 @@ only useful to override this for testing purposes.>
 The profile name in the configuration file (F<~/.aws/config> or
 F<~/.aws/credentials>).
 
- my $aws_creds = new Amazon::Credentials({ profile => 'sandbox' });
+ my $aws_creds = Amazon::Credentials->new({ profile => 'sandbox' });
 
 The class will also look for the environment variable C<AWS_PROFILE>,
 so you can invoke your script like this:
@@ -124,7 +125,7 @@ default:  env, role, container, file
 
 Example:
 
-  my $creds = new Amazon::Credentials( { order => [ qw/file env role/] });
+  my $creds = Amazon::Credentials->new( { order => [ qw/file env role/] });
 
 =over 10
 
@@ -159,7 +160,7 @@ credentials by passing it into the constructor or setting it the
 environment variable C<AWS_PROFILE>.  If no profile is provided, the
 default credentials or the first profile found is used.
 
- my $aws_creds = new Amazon::Credentials({ order => [qw/environment role file/] });
+ my $aws_creds = Amazon::Credentials->new({ order => [qw/environment role file/] });
 
 =item container - Task Role
 
@@ -205,12 +206,9 @@ sub new {
   }
 
   unless ($self->get_user_agent) {
-    $self->set_user_agent(new LWP::UserAgent);
+    $self->set_user_agent(LWP::UserAgent->new());
   }
 
-  $self->get_logger->debug(Dumper [ $self->get_profile() ])
-    if $self->get_debug;
-
   $self->set_profile($ENV{AWS_PROFILE})
     unless $self->get_profile;
 
@@ -247,7 +245,7 @@ sub get_default_region {
   # executing on an EC2 or container.
   my $url = AWS_AVAILABILITY_ZONE_URL;
   
-  my $ua = ref($self) ? $self->get_user_agent : new LWP::UserAgent;
+  my $ua = ref($self) ? $self->get_user_agent : LWP::UserAgent->new();
 
   my $req = HTTP::Request->new( GET => $url );
      
@@ -255,7 +253,7 @@ sub get_default_region {
     my $rsp = $ua->request($req);
       
     # if not 200, then get out of Dodge
-    die "could not get availability zone\n"
+    croak "could not get availability zone\n"
       unless $rsp->is_success;
     
     my $region = $rsp->content;
@@ -272,13 +270,14 @@ sub set_credentials {
   my $creds = shift || $self->get_ec2_credentials();
   
   if ( $creds->{aws_secret_access_key} && $creds->{aws_access_key_id} ) {
+    
     $self->set_aws_secret_access_key($creds->{aws_secret_access_key});
     $self->set_aws_access_key_id($creds->{aws_access_key_id});
     $self->set_token($creds->{token});
     $self->set_expiration($creds->{expiration});
   }
   else {
-    die "no credentials available\n";
+    croak "no credentials available\n";
   }
 }
 
@@ -345,7 +344,8 @@ sub find_credentials {
 	local $CWD = home;
 	next unless -e $config;
 	
-	open my $fh, "<$config" or die "could not open credentials file!";
+	open my $fh, '<', $config
+          or croak 'could not open credentials file!';
 	
 	my $last_profile = '';
 	my $profile_to_find = $profile_name;
@@ -441,6 +441,8 @@ sub is_token_expired {
   if ( defined $expiration_date ) {
     # AWS recommends getting credentials 5 minutes prior to expiration
     my $g = _iso8601_to_time($expiration_date);
+    
+    $self->get_logger->debug("gmt time in seconds: $g\n");
 
     # shave 5 minutes or window interval off of the expiration time
     $g -= $window_interval * 60;
@@ -449,6 +451,7 @@ sub is_token_expired {
     my $seconds_left = $g - time;
 
     if ( $self->get_debug ) {
+      $self->get_logger->debug("seconds left : $seconds_left\n");
       my $hours = int($seconds_left/3600);
       my $minutes = int(($seconds_left - $hours * 3600)/60);
       my $seconds = $seconds_left - ($hours * 3600 + $minutes * 60);
@@ -593,7 +596,7 @@ sub refresh_token {
   
   if ( $self->get_container && $self->get_container eq 'ECS' ) {
     $creds = $self->get_creds_from_container;
-   }
+  }
   elsif ( $self->get_role ) {
     $creds = $self->get_creds_from_role;
   }
@@ -601,53 +604,85 @@ sub refresh_token {
   $self->get_logger->debug(Dumper [$creds])
     if $self->get_debug;
 
-  die "unable to refresh token!"
-    unless ref($creds);
+  croak 'unable to refresh token!'
+    if !ref($creds) || !keys %$creds;
   
   $self->set_credentials($creds);
 }
 
 
+=pod
+
+=head2 get_creds_from_container
+
+ get_creds_from_container()
+
+Retrieves credentials from the container's metadata at
+http://169.254.170.2.  Returns a hash of credentials containing:
+
+  aws_access_key_id                                                                                                                             
+  aws_secret_access_key
+  aws_session_token
+
+Returns an empty hash if no credentials found.  The environment
+variable C<AWS_CONTAINER_CREDENTIALS_RELATIVE_URI> must exist or you
+must pass the value of the path as an argument.
+
+=cut
+
 sub get_creds_from_container {
-  my $self = shift;
+  my ($self, $uri) = @_;
+
+  $uri = $uri || $ENV{AWS_CONTAINER_CREDENTIALS_RELATIVE_URI};
 
   my $creds = {};
-  
-  if ( exists $ENV{AWS_CONTAINER_CREDENTIALS_RELATIVE_URI} ) {
-    # try to get credentials from instance role
-    my $url = sprintf("%s%s", AWS_CONTAINER_CREDENTIALS_URL, $ENV{AWS_CONTAINER_CREDENTIALS_RELATIVE_URI});
-    
-    my $ua = $self->get_user_agent;
-    my $req = HTTP::Request->new( GET => $url );
-    $req->header("Accept", "*/*");
-    
-    $self->get_logger->debug(Dumper [ "HTTP REQUEST:\n", $req ])
-      if $self->get_debug;
 
-    $self->get_logger->debug(Dumper [ $req->as_string ])
-      if $self->get_debug;
-    
-    my $rsp = $ua->request($req);
-    
-    $self->get_logger->debug(Dumper [ "HTTP RESPONSE:\n", $rsp ])
-      if $self->get_debug;
-    
-    # if not 200, then get out of Dodge
-    if ( $rsp->is_success ) {
-      $creds->{serialized} = $rsp->content;
-      my $this = from_json($rsp->content);
-      @{$creds}{qw/source container aws_access_key_id aws_secret_access_key token expiration/} =
-	('IAM','ECS', @{$this}{qw/AccessKeyId SecretAccessKey Token Expiration/});
-    }
-    else {
-      $self->get_logger->debug( "return code: " . $rsp->status_line . "\n");
-    }
-    
+  if ($uri) {
+    eval {
+      # try to get credentials from instance role
+      my $url = AWS_CONTAINER_CREDENTIALS_URL . $uri;
+
+      my $ua = $self->get_user_agent;
+      my $req = HTTP::Request->new( GET => $url );
+      $req->header( "Accept", "*/*" );
+
+      $self->get_logger->debug( Dumper [ "HTTP REQUEST:\n", $req ] )
+        if $self->get_debug;
+
+      $self->get_logger->debug( Dumper [ $req->as_string ] )
+        if $self->get_debug;
+
+      my $rsp = $ua->request($req);
+
+      $self->get_logger->debug( Dumper [ "HTTP RESPONSE:\n", $rsp ] )
+        if $self->get_debug;
+
+      # if not 200, then get out of Dodge
+      if ( $rsp->is_success ) {
+
+        $creds->{serialized} = $rsp->content;
+
+        my $this = from_json( $rsp->content );
+
+        @{$creds}{
+          qw/source container aws_access_key_id aws_secret_access_key token expiration/
+          } = (
+          'IAM', 'ECS',
+          @{$this}{qw/AccessKeyId SecretAccessKey Token Expiration/}
+          );
+      } ## end if ( $rsp->is_success )
+      else {
+        $self->get_logger->debug(
+          "return code: " . $rsp->status_line . "\n" );
+      }
+    };
+
     $creds->{error} = $@ if $@;
-  }
-  
-  $creds;
-}
+
+  } ## end if ($uri)
+
+  return $creds;
+} ## end sub get_creds_from_container
 
 =pod
 
